#!/usr/bin/env python3

import os, glob, jinja2
from shutil import copyfile


def render(tpl_path, context):
    path, filename = os.path.split(tpl_path)
    return jinja2.Environment(undefined=jinja2.StrictUndefined,
        loader=jinja2.FileSystemLoader(path or './')
    ).get_template(filename).render(context)


rankmap = {1:'vector', 2:'matrix', 3:'tensor', 4:'four_dimensional'}

def gen_interface_list(types):
    intrfc_list = []
    for t in types:
        for rank in (1,2,3,4):
            intrfc_list.append({
                      'vtype'     : t,
                      'rank'      : rank,
                      'vrankname' : rankmap[rank],
                      'subname'   : t[0]+rankmap[rank][0],
                      'vsize'     : rankmap[rank][0]+'size',
                      'vsizedims' : "({})".format(rank),
                      'ranks'     : ",".join([':']*rank)
                      })
    return intrfc_list


def gen_module(context):
    fname = 'devxlib_buffers.jf90'
    bname, _ = os.path.splitext(fname)
    with open('devxlib_buffer.f90','w') as f:
        f.write(render(fname, context))

def gen_module2(context):
    fname = 'devxlib_buffers.jf90'
    bname, _ = os.path.splitext(fname)
    with open('devxlib_pinned.f90','w') as f:
        f.write(render(fname, context))

def main():
    # clean up
    #open('devxlib_buffer.f90','w').close()

    # Types used in the interface
    interfaces = gen_interface_list(['integer', 'real(real64)', 'complex(real64)'])
    context = {'interfaces'  : interfaces,
               'cudamod'     : 'use cudafor',
               'ompmod'      : 'use omp_lib',
               'attributes'  : 'device',
               'pointer_type': 'c_devptr',
               'modulename'  : 'devxlib_buffer',
               'typename'    : 'devxlib_buffer_t'}
    gen_module(context)

#    context = {'interfaces'  : interfaces,
#               'cudamod'     : 'use cudafor',
#               'ompmod'      : 'use omp_lib',
#               'attributes'  : '',
#               'pointer_type': 'c_ptr',
#               'modulename'  : 'devxlib_pinned',
#               'typename'    : 'devxlib_pinned_t'}
#    gen_module(context)

def main2():
    # clean up
    #open('devxlib_pinned.f90','w').close()

    # Types used in the interface
    interfaces = gen_interface_list(['integer', 'real(real64)', 'complex(real64)'])
    context = {'interfaces'  : interfaces,
               'cudamod'     : 'use cudafor',
               'ompmod'      : 'use omp_lib',
               'attributes'  : '',
               'pointer_type': 'c_ptr',
               'modulename'  : 'devxlib_pinned',
               'typename'    : 'devxlib_pinned_t'}
    gen_module2(context)

    ## set multiple extensions
    #copyfile("device_fbuff.f90","device_fbuff.F")


if __name__ == "__main__":
    main()
    main2()



#!/usr/bin/env python3

import sys, os, jinja2
from shutil import copyfile

def render(tpl_path, context):
    path, filename = os.path.split(tpl_path)
    return jinja2.Environment(undefined=jinja2.StrictUndefined,
        loader=jinja2.FileSystemLoader(path or './')
    ).get_template(filename).render(context)

####

types_l=['real', 'complex', 'integer', 'logical' ]

sp_d={'name':'sp', 'val':'real32'}
dp_d={'name':'dp', 'val':'real64'}
i4_d={'name':'i4', 'val':'int32'}
i8_d={'name':'i8', 'val':'int64'}
l4_d={'name':'l4', 'val':'int32'}

kinds_d={'real':    [sp_d, dp_d] ,
         'complex': [sp_d, dp_d] ,
         'integer': [i4_d, i8_d] ,
         'logical': [l4_d] }

nranks=6
nranks_tests=4

# Generate module
with open('devxlib_memcpy.f90', 'w') as f:
    f.write(render('devxlib_memcpy.jf90',
                    {'types' : types_l, 'kinds' : kinds_d, 'dimensions': nranks }
                  ))

# Generate submodules
with open('devxlib_memcpy_d2d.f90', 'w') as f:
    f.write(render('devxlib_memcpy_d2d.jf90',
                    {'types' : types_l, 'kinds' : kinds_d, 'dimensions': nranks }
                  ))
with open('devxlib_memcpy_h2h.f90', 'w') as f:
    f.write(render('devxlib_memcpy_h2h.jf90',
                    {'types' : types_l, 'kinds' : kinds_d, 'dimensions': nranks }
                  ))
with open('devxlib_memcpy_h2d.f90', 'w') as f:
    f.write(render('devxlib_memcpy_h2d.jf90',
                    {'types' : types_l, 'kinds' : kinds_d, 'dimensions': nranks }
                  ))
with open('devxlib_memcpy_d2h.f90', 'w') as f:
    f.write(render('devxlib_memcpy_d2h.jf90',
                    {'types' : types_l, 'kinds' : kinds_d, 'dimensions': nranks }
                  ))

# Generate tests
with open('test_memcpy.f90', 'w') as f:
    f.write(render('test_memcpy.jf90',
                    {'types' : types_l, 'kinds' : kinds_d, 'dimensions': nranks_tests }
                  ))

with open('test_memcpy_async.f90', 'w') as f:
    f.write(render('test_memcpy_async.jf90',
                    {'types' : types_l, 'kinds' : kinds_d, 'dimensions': nranks_tests }
                  ))

## set multiple extensions
#copyfile("device_memcpy.f90","device_memcpy.F")
#copyfile("device_memcpy_interf.f90","device_memcpy_interf.F")




#!/usr/bin/env python3

import sys, os, jinja2
from shutil import copyfile

def render(tpl_path, context):
    path, filename = os.path.split(tpl_path)
    return jinja2.Environment(undefined=jinja2.StrictUndefined,
        loader=jinja2.FileSystemLoader(path or './')
    ).get_template(filename).render(context)

####

types_l=['real', 'complex']
nranks=6
kinds_l= [{'name':'dp', 'val':'real64'},
          {'name':'sp', 'val':'real32'}]

# Generate module
with open('devxlib_auxfunc.f90', 'w') as f:
    f.write(render('devxlib_auxfunc.jf90',
                    {'types' : types_l, 'dimensions': nranks, 'precision' : kinds_l }
                  ))

# Generate submodules
with open('devxlib_auxfunc_vec_upd.f90', 'w') as f:
    f.write(render('devxlib_auxfunc_vec_upd.jf90',
                    {'types' : types_l, 'dimensions': nranks, 'precision' : kinds_l }
                  ))
with open('devxlib_auxfunc_mat_upd.f90', 'w') as f:
    f.write(render('devxlib_auxfunc_mat_upd.jf90',
                    {'types' : types_l, 'dimensions': nranks, 'precision' : kinds_l }
                  ))
with open('devxlib_auxfunc_addscal.f90', 'w') as f:
    f.write(render('devxlib_auxfunc_addscal.jf90',
                    {'types' : types_l, 'dimensions': nranks, 'precision' : kinds_l }
                  ))
with open('devxlib_auxfunc_conjg.f90', 'w') as f:
    f.write(render('devxlib_auxfunc_conjg.jf90',
                    {'types' : types_l, 'dimensions': nranks, 'precision' : kinds_l }
                  ))

## Generate tests
#with open('test_auxfunc.f90', 'w') as f:
#    f.write(render('test_auxfunc.jf90',
#                    {'types' : types_l, 'dimensions': nranks, 'precision' : kinds_l }
#                  ))

## set multiple extensions
#copyfile("device_auxfunc.f90","device_auxfunc.F")
#copyfile("device_auxfunc_interf.f90","device_auxfunc_interf.F")

